/*
* This Source Code Form is subject to the terms of the Mozilla Public
* License, v. 2.0. If a copy of the MPL was not distributed with this file,
* You can obtain one at http://mozilla.org/MPL/2.0/.
*
* Copyright (c) 2020, Krunoslav Velfel, kvelfel <at> gmail <dot> com
*/
public class Tabla {
	public static enum igrac {
		IKS,
		OKS
	}
	
	public static enum rezultat {
		POBIJEDIO_IKS,
		POBIJEDIO_OKS,
		NERIJESENO,
		IGRA_U_TOKU
	}

	private final int TABELA_SIZE = 3;
	private final int NDEF = 0;
	private final int IKS = 1;
	private final int OKS = -1;
	
	private int brojPoteza = TABELA_SIZE * TABELA_SIZE;
	private int [][] tabela = new int[TABELA_SIZE][TABELA_SIZE];
	private igrac naPotezu;
	
	public Tabla(Tabla.igrac naPotezu) {
		for(int i = 0; i < TABELA_SIZE; i++) {
			for(int j = 0; j < TABELA_SIZE; j++) {
				tabela[i][j] = NDEF;
			}
		}
		this.naPotezu = naPotezu;
	}

	public rezultat potez(int poljeX, int poljeY) {
		if(poljeX < 0 || poljeY < 0 || poljeX >= TABELA_SIZE || poljeY >= TABELA_SIZE) {
			System.err.printf("GREŠKA: koordinate (%d, %d) nijesu validne.\n", poljeX, poljeY);
			return rezultat.IGRA_U_TOKU;
		}

		if(tabela[poljeX][poljeY] != NDEF) {
			System.err.printf("GREŠKA: polje (%d, %d) je zauzeto.\n", poljeX, poljeY);
			return rezultat.IGRA_U_TOKU;
		}

		if(naPotezu == igrac.IKS) {
			tabela[poljeX][poljeY] = IKS;
		} else {
			tabela[poljeX][poljeY] = OKS;
		}

		/* Provjeri po kolonama: */
		for(int i = 0; i < TABELA_SIZE; i++) {
			int suma = 0;
			for(int j = 0; j < TABELA_SIZE; j++) {
				suma += tabela[i][j];
			}
			if(suma == TABELA_SIZE * IKS) {
				return rezultat.POBIJEDIO_IKS;
			} else if(suma == TABELA_SIZE * OKS) {
				return rezultat.POBIJEDIO_OKS;
			}
		}
		
		/* Provjeri po redovima: */
		for(int j = 0; j < TABELA_SIZE; j++) {
			int suma = 0;
			for(int i = 0; i < TABELA_SIZE; i++) {
				suma += tabela[i][j];
			}
			if(suma == TABELA_SIZE * IKS) {
				return rezultat.POBIJEDIO_IKS;
			} else if(suma == TABELA_SIZE * OKS) {
				return rezultat.POBIJEDIO_OKS;
			}
		}

		/* Provjeri dijagonalno: */
		int sumaGlavneDijagonale = 0;
		int sumaSporedneDijagonale = 0;

		for(int i = 0; i < TABELA_SIZE; i++) {
			sumaGlavneDijagonale += tabela[i][i];
			sumaSporedneDijagonale += tabela[TABELA_SIZE - 1 - i][i];
		}
		
		if((sumaGlavneDijagonale == TABELA_SIZE * IKS) || (sumaSporedneDijagonale == TABELA_SIZE * IKS)) {
			return rezultat.POBIJEDIO_IKS;
		} else if((sumaGlavneDijagonale == TABELA_SIZE * OKS) || (sumaSporedneDijagonale == TABELA_SIZE * OKS)) {
			return rezultat.POBIJEDIO_OKS;
		}

		/* Da li je uopšte preostalo slobodnih poteza? */
		brojPoteza--;
		if(brojPoteza == 0) {
			return rezultat.NERIJESENO;
		}

		/* Ako nema pobjednika i nije kraj partije, red je na protivnika da odigra sljedeći potez: */
		if(naPotezu == igrac.IKS) {
			naPotezu = igrac.OKS;
		} else {
			naPotezu = igrac.IKS;
		}

		return rezultat.IGRA_U_TOKU;
	}
	
	public Tabla.igrac getNaPotezu() {
		return naPotezu;
	}
}